/*
 * <N_OCO_COPYRIGHT>
 * Licensed Materials - Property of IBM
 * 
 * 5724-H72, 5655-R36, 5724-L26, 5655-L82     
 * 
 * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * <NOC_COPYRIGHT>
 
DISCLAIMER
All source code and/or binaries attached to this document are referred 
to here as "the Program". IBM is not providing program services of any 
kind for the Program. IBM is providing the Program on an "AS IS" basis 
without warranty of any kind. IBM WILL NOT BE LIABLE FOR ANY ACTUAL, 
DIRECT, SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC 
CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF IBM, 
OR ITS RESELLER, HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 

 */

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import com.ibm.msg.client.jms.JmsConnectionFactory;
import com.ibm.msg.client.jms.JmsFactoryFactory;
import com.ibm.msg.client.wmq.WMQConstants;

/**
 * Syntax: java SampleJMSMsgProperty [color]
 * Examples:
 *   To produce a message with out the message property "color" 
 *     java SampleJMSMsgProperty
 *   To produce a message with the message property "color='red'" 
 *     java SampleJMSMsgProperty red
 *   To produce a message with the message property "color='blue'" 
 *     java SampleJMSMsgProperty blue
 *
 * A minimal and simple application for Point-to-point and Pub/Sub messaging.
 * It uses a Java message property: color
 * It is based on:
 * C:\Program Files\IBM\WebSphere MQ\tools\jms\samples\simple\SimplePTP.java
 * 
 ** Customization:
 * To keep the sample as simple as possible,
 * This sample makes use of fixed literals, 
 * any customisations will require re-compilation of this source file. 
 * You will need to modify the corresponding statements to suit your needs.
      cf.setStringProperty(WMQConstants.WMQ_HOST_NAME, "localhost");
      cf.setIntProperty(WMQConstants.WMQ_PORT, 1424);
      cf.setStringProperty(WMQConstants.WMQ_CHANNEL, "SYSTEM.DEF.SVRCONN");
      cf.setStringProperty(WMQConstants.WMQ_QUEUE_MANAGER, "QMPS");
      * For queue:
      destination = session.createQueue("queue:///Q1");
      * For topic:
      destination = session.createTopic("topic://sales");

 * 
 * Notes:
 * API type: JMS API (v1.1, unified domain)
 * Messaging domain: Point-to-point and Pub/Sub
 * Provider type: IBM MQ
 * Connection mode: Client connection
 * JNDI in use: No
 * 
 */
public class SampleJMSMsgProperty {

  // System exit status value (assume unset value to be 1)
  private static int status = 1;

  /**
   * Main method
   * 
   * @param args
   */
  public static void main(String[] args) {

    // Variables
    String color = null;
    Connection connection = null;
    Session session = null;
    Destination destination = null;
    MessageProducer producer = null;
    MessageConsumer consumer = null;

    switch (args.length) {
    case 1:
    	color = args[0];
    
    case 0:
    	break;
    
    default:
    	System.out.println("syntax: SampleJMSMsgProperty [color]");
    	System.exit(1);
    }

    try {
      // Create a connection factory
      JmsFactoryFactory ff = JmsFactoryFactory.getInstance(WMQConstants.WMQ_PROVIDER);
      JmsConnectionFactory cf = ff.createConnectionFactory();

      // Set the properties
      cf.setStringProperty(WMQConstants.WMQ_HOST_NAME, "localhost");
      cf.setIntProperty(WMQConstants.WMQ_PORT, 1424);
      cf.setStringProperty(WMQConstants.WMQ_CHANNEL, "SYSTEM.DEF.SVRCONN");
      cf.setIntProperty(WMQConstants.WMQ_CONNECTION_MODE, WMQConstants.WMQ_CM_CLIENT);
      cf.setStringProperty(WMQConstants.WMQ_QUEUE_MANAGER, "QMPS");

      // Create JMS objects
      connection = cf.createConnection();
      session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
//      destination = session.createQueue("queue:///Q1");
      destination = session.createTopic("topic://sales");
      producer = session.createProducer(destination);

      long uniqueNumber = System.currentTimeMillis() % 1000;
      TextMessage message = session.createTextMessage("Text created by SampleJMSMsgProperty");
      
      if (color == null) {
      	System.out.println("* Creating message with no color");
      } else {
      	System.out.println("* Creating message with color " + color);
      	message.setStringProperty("color", color);
      }

      // Start the connection
      connection.start();

      // And, send the message
      producer.send(message);
      System.out.println("Sent message:\n" + message);

      recordSuccess();
    }
    catch (JMSException jmsex) {
      recordFailure(jmsex);
    }
    finally {
      if (producer != null) {
        try {
          producer.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Producer could not be closed.");
          recordFailure(jmsex);
        }
      }

      if (session != null) {
        try {
          session.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Session could not be closed.");
          recordFailure(jmsex);
        }
      }

      if (connection != null) {
        try {
          connection.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Connection could not be closed.");
          recordFailure(jmsex);
        }
      }
    }
    System.exit(status);
  } // end main()

  /**
   * Process a JMSException and any associated inner exceptions.
   * 
   * @param jmsex
   */
  private static void processJMSException(JMSException jmsex) {
    System.out.println(jmsex);
    Throwable innerException = jmsex.getLinkedException();
    if (innerException != null) {
      System.out.println("Inner exception(s):");
    }
    while (innerException != null) {
      System.out.println(innerException);
      innerException = innerException.getCause();
    }
  }

  /**
   * Record this run as successful.
   */
  private static void recordSuccess() {
    System.out.println("SUCCESS");
    status = 0;
  }

  /**
   * Record this run as failure.
   * 
   * @param ex
   */
  private static void recordFailure(Exception ex) {
    if (ex != null) {
      if (ex instanceof JMSException) {
        processJMSException((JMSException) ex);
      }
      else {
        System.out.println(ex);
      }
    }
    System.out.println("FAILURE");
    status = -1;
  }

}
